/************************************************************************************************

 Author        : ADI - Apps            www.analog.com/AduC7060

 Date          : November 2007

 File          : I2C_Master.c

 Hardware      : ADuC706x

 Description   : This I2C Master example acts as interface between the PC based application I2CSerial.exe
 				and the Slave EPROM. 
				This application accepts commands via the serial port from the PC, converts these to I2C
				and passes them to the Slave/EPROM device.
				 Using the command window based executable, I2CSerial.exe, data bytes
				are sent serially to the Master via UART. Depending on whether	the application sent
				a write or read command, the Master sends the appropriate command to the Slave via 
				I2C channel. If a slave read is requested, the Master will request the data
				from the I2Cserial.exe specified address and return the data to the PC.

*************************************************************************************************/
// Bit Definitions
#define BIT0  0x01
#define BIT1  0x02
#define BIT2  0x04
#define BIT3  0x08
#define BIT4  0x10
#define BIT5  0x20
#define BIT6  0x40
#define BIT7  0x80
#define BIT8  0x100
#define BIT9  0x200
#define BIT10 0x400
#define BIT11 0x800
#define BIT12 0x1000
#define BIT13 0x2000
#define BIT14 0x4000
#define BIT15 0x8000
#define I2C_RD_BIT 0x1

#include<aduc7060.h>
// Function Definitions
void Uart_Packet_Handler( volatile unsigned int value, volatile unsigned char Byte_Number);
unsigned char I2CWritePacket( unsigned int length);
unsigned char I2CReadPacket( unsigned int length);
void delay(int);


// Application specific variables
volatile unsigned int rxDat[256];	  // 256 byte buffer-array for I2C receive data
volatile unsigned char ucRxCount = 0; // rxDat array control variable.
volatile unsigned char ucComRx = 0;   // Variable used to read UART Rx buffer contents into 
volatile unsigned char ucMsgType = 0; // Indicates I2CSerial.exe application requested I2C read or write
volatile unsigned int ucLength = 0;  // Indicates I2CSerial.exe "number of bytes" for I2C read or write
volatile unsigned int ucStartAddress = 0;	// Indicates I2CSerial.exe "Start address" for I2C read or write
volatile unsigned char ucSendI2CPacket = 0;	// Used in main loop to indicate I2C message is ready for transmission
volatile unsigned char ucTransfer[512];	 // Buffer used during I2C "writes to Slave"	
volatile unsigned int ucRdCnt = 0;	 // Counter for UART receive messages
volatile unsigned char ucDataCnt = 0; // Message Parser counter for UART messages
volatile unsigned char ucCOMSTA0 = 0; // Variable used to store COMSTA0, UART status register
volatile unsigned char ucCOMIID0 = 0; // Variable used to store COMIID0, UART Interrupt status register	 
unsigned int uiIRQSTA = 0;			  // Variable used to store IRQSTA, UART interrupt status register
volatile unsigned char ucTxBufferEmpty = 0;	// Indicates UART Tx buffer empty
volatile unsigned char ucTransmitInProgress =0; // Indicates I2C Tx FIFO ready for data
volatile unsigned char ucReceiveInProgress = 0; // Indicates I2C Rx sequence with the SLave is complete
volatile unsigned char ucI2CWrCnt = 0; // Counter for controlling I2C writes
volatile unsigned char ucI2CNT = 0;	   // The 4 following variables handle I2C reads of greater than 8 bytes
volatile unsigned char ucI2CNTReload = 0;
volatile unsigned char ucNumReloads =0;	
volatile unsigned char ucRemainder =0;	
volatile unsigned char ucValidUARTPacket = 0;  // Indicates a valid UART packet has been received.
volatile unsigned char ucRdLength = 0;	   // Stores number of bytes to be read from Slave via I2C
unsigned int ucBytesToRD = 0;			// Stores number of bytes to be read from Slave via I2C
volatile unsigned char ucToggleCnt = 0;
volatile unsigned char ucStringCnt = 0;
volatile unsigned char ucLoadVal = 0;
volatile unsigned char ucLoadCnt = 0;
volatile unsigned char StartByteGone = 0;
volatile unsigned char ucTest = 0;

unsigned int uiTest = 0;
int main(void)
{
    POWKEY1 = 0x1;
	POWCON0 = 0x78;		   // Set core to max CPU speed of 10.24Mhz
	POWKEY2 = 0xF4;


	StartByteGone = 0;
	// Configure P0.1 and P0.3 for I2C mode. Configure P1.0 and P1.1 for UART mode. 
	GP0CON0 = BIT4 + BIT12; // Select SPI/I2C alternative function for P0.1 & P0.3
	GP0KEY1 = 0x7;		   // Write to GP0KEY1
	GP0CON1	= BIT1;		   // Select I2C functionality for P0.1 & P0.3
	GP0KEY2 = 0x13;		   // Write to GP0KEY2

	GP1CON = BIT0 + BIT4;  // Select UART functionality for P1.0/P1.1

	// Enable I2C Master mode, baud rate and interrupt sources
	I2CMCON = BIT0 + BIT4  // Enable I2C Master + Enable Rx interrupt
			+ BIT5 + BIT7 + BIT8; // Enable Tx interrupt + Enable transmission complete interrupt
//	I2CDIV  = 0x3131;	   // Select 100.3kHz clock rate
	I2CDIV  = 0x2424;	   // Select 135 kHz clock rate
	IRQEN = BIT15 + BIT11; // Enable I2C Master and UART interrupts

	// Initialize the UART for 9600-8-N
	COMCON0 = BIT7;			// Enable access to COMDIV registers
	COMDIV0 = 0x21;
	COMDIV1 = 0x00;

	COMCON0 = BIT0 + BIT1 + BIT2;
	COMIEN0 = BIT0 + BIT1;	 // Enable UART interrupts when Rx full and Tx buffer empty.
	while (1)
	{
	
		if (ucSendI2CPacket == 1)
		{
			if (ucMsgType == 0)
			{
			 	I2CWritePacket(ucLength);
				ucSendI2CPacket = 0;
				ucDataCnt = 0;
				ucRdCnt = 0;
			}
			else
			{
				I2CReadPacket(ucLength);
				ucSendI2CPacket = 0;
				ucDataCnt = 0;
				ucRdCnt = 0;
				ucLoadCnt = 0;
			}
		}	
	}
}

// This Function Parses the UART packets received from the PC application.
void Uart_Packet_Handler( volatile unsigned int value, volatile unsigned char Byte_Number)
{
  switch(Byte_Number)
  {
	   case 1:	//Read/Write indicator
	   	{
			ucMsgType = value;
			break;
		} 
	   case 2:  //	Length of data in packet
	   {
	   		ucLength = value;
			break;
	   }	   
	   case 6 : // Memory Start Address
	   {
	   		ucStartAddress = value;
			break;
	   }
	   case 9 : // Memory Start Address
	   {
	   		if (ucMsgType == 1)	// if msg type is a read - reset buffer
			{
				ucDataCnt = 0;	// Reset message parse register
				ucSendI2CPacket = 1;  //Send Packet to the slave
				ucRdCnt = 0;
				ucValidUARTPacket = 0;
			}
			break;
	   }
		default :
	   break;
	}
}
// This function Starts an I2C Write sequence
unsigned char I2CWritePacket(unsigned int length )
{
	// Transmit
	ucTransmitInProgress = 1;
	I2CFSTA = BIT9;			// Flush Master Tx FIFO
	I2CFSTA &= ~BIT9;
	I2CADR0 = 	0xA0;	 			// set i2c address	 (LSB = 0, Master Write)
	I2CMTX  =   ucStartAddress;				// send i2c byte - register Address 0x7

	ucI2CWrCnt = 0;
	while (ucTransmitInProgress == 1)
	{}
	
	return 1;
}

// Fucntion used to commence I2C Read sequence
unsigned char I2CReadPacket( unsigned int length)
{
	unsigned char n = 0;
	

	I2CMCNT0 = ucLength - 1; 	  // Load I2CMCNT0 with the number of bytes to be read from Slave
	ucBytesToRD = I2CMCNT0;
	// Transmit
	ucTransmitInProgress = 1;
	StartByteGone = 1;
	I2CFSTA = BIT9; // Flush Master Tx FIFO
	I2CFSTA &= ~BIT9; 
	ucLength = 0;

	IRQCLR = BIT15;
	I2CADR0 = 	0xA0;	 			// set i2c address	 (LSB = 0, Master Write)

	I2CMTX =   ucStartAddress;		// send i2c register Address 0x7
	while ((I2CMSTA  & 0x3) != 0x0)
	{}
	// Repeated Start Condition!
	I2CADR0 = 	0xA0 + I2C_RD_BIT;	// set i2c address	+ (LSB = 1, Master Read)
	IRQEN |= BIT15;
	
		
	ucReceiveInProgress = 1;

	while (ucReceiveInProgress == 1) {}
	 
	delay (0x12000);
	I2CFSTA = BIT9;			// Flush Master Tx FIFO
	I2CFSTA &= ~BIT9;
	ucRdLength = 0;
	ucDataCnt = 0;
	ucRdCnt = 0;
	ucTxBufferEmpty = 0;
	COMTX = rxDat[0];
	ucSendI2CPacket = 0;
	for (n = 1; n < ucBytesToRD +1 ; n++)
	{
		while (ucTxBufferEmpty != 1){}
		ucTxBufferEmpty = 0;
		COMTX = rxDat[n];
	}
	return 1;
}
// Simple Delay routine
void delay (int length)
{
	while (length >0)
    	length--;
}
/*************************************************/
/*************************************************/
/************	IRQ Service Routine  *************/
/*************************************************/
/*************************************************/
void IRQ_Handler(void) __irq
{
	unsigned long IRQSTATUS = 0;
	unsigned int I2CMSTATUS = 0;

	IRQSTATUS = IRQSTA;	   // Read off IRQSTA register
	I2CMSTATUS = I2CMSTA;
// UART ISR
	if ((IRQSTATUS & BIT11) == BIT11)	//UART interrupt source
	{
		ucCOMSTA0 = COMSTA0;
		ucCOMIID0 = COMIID0;
		if ((ucCOMIID0 & 0x2) == 0x2)	  // Transmit buffer empty
		{
		  ucTxBufferEmpty = 1;
		}
		
		if ((ucCOMIID0 & 0x4) == 0x4)	  // Receive byte
		{
			ucComRx	= COMRX;
			if 	((ucRdCnt == 0) && (ucComRx == 8))	 // Received Start character 0x08
				ucValidUARTPacket = 1;
			if ((ucRdCnt < 10) && (ucSendI2CPacket == 0) && (ucValidUARTPacket == 1))
			{					
				Uart_Packet_Handler( ucComRx, ucRdCnt);
				ucRdCnt++;
			}
			else if ((ucSendI2CPacket == 0) && (ucValidUARTPacket == 1))
			{
				ucTransfer[ucDataCnt++] = COMRX;				 
				if (ucDataCnt == ucLength)
				{
			  		ucDataCnt = 0;	// Reset message parse register
					ucSendI2CPacket = 1;  //Send Packet to the slave
					ucValidUARTPacket = 0;
				}
			}
		}
	}
// I2C Master ISR
	if ((IRQSTATUS & BIT15) == BIT15)	//If I2C Master interrupt source
	{
	   // Transmit
		if((I2CMSTATUS & 0x4) == 0x4)  	// Master Transmit IRQ	
		{											    
			if (ucI2CWrCnt < ucLength)
				I2CMTX = ucTransfer[ucI2CWrCnt++];	// Tx FIFO ready for another byte
			else 
			{
					ucTransmitInProgress = 0;
					ucRxCount = 0; 
					ucI2CNT = 0;
			}
		}	
		// Receive
		if((I2CMSTATUS & 0x8) == 0x8)				    // Master Recieve IRQ
		{
			rxDat[ucRxCount++] = I2CMRX;

			if(ucRxCount == I2CMCNT0)
				ucReceiveInProgress = 0;	
			ucI2CNT++;

		}
		if((I2CMSTATUS & 0x100) == 0x100)			    // Transmission Complete/Stop detected
		{
			uiTest++;					
			
		}
		if((I2CMSTATUS & 0x80) == 0x80)			    // NACK Data detected
		{
			uiTest++;					
			
		}	
		if((I2CMSTATUS & 0x10) == 0x10)			    // NACK Address detected
		{
			uiTest++;					
			
		}	
	}
}


